#!/usr/bin/env python
# -*- coding: utf-8 -*-
##############################################################################
#    
#    OpenERP, Open Source Enterprise Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://openerp.com>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.     
#
##############################################################################

"""Script that generates a relationship graph for an OpenERP module."""

import optparse
import xmlrpclib
import base64

URL_PATTERN = 'http://%s:%s/xmlrpc/%s'

def execute(*args):
    """Execute a method on an object through the proxy."""
    return proxy.execute(db, uid, pwd, *args)
    
parser = optparse.OptionParser('usage: %prog host port db login pwd module')
parser.add_option('--level', dest='level', help='Number of times a model is processed when it is in relation with another model')
parser.add_option('--models_file', dest='models_file', help='Text file where each line is a model name')
(options, args) = parser.parse_args()
if len(args) != 6:
    parser.error('incorrect number of arguments')
host, port_arg, db, login, pwd, module = args
try:
    port = int(port_arg)
except ValueError: 
    parser.error('%s is not a number' % port_arg)
common_url = URL_PATTERN % (host, port, 'common')
proxy_common = xmlrpclib.ServerProxy(common_url)
uid = proxy_common.login(db, login, pwd)
if uid:
    print 'Logged in as user %s with uid %d' % (login, uid)
else:
    print 'Could not login'
url = URL_PATTERN % (host, port, 'object') 
proxy = xmlrpclib.ServerProxy(url)
module_ids = execute('ir.module.module', 'search', [('name', '=', module)])
old_values = execute('ir.module.module', 'read', module_ids, ['level', 'filter_model_ids'])[0]
old_values['filter_model_ids'] = [(6, 0, old_values['filter_model_ids'])]
del old_values['id']
new_values = {}
new_values['level'] = options.level or 1
if options.models_file:
    models_file = open(options.models_file, 'r')
    models = models_file.read().splitlines()
    model_ids = execute('ir.model', 'search', [('model', 'in', models)])
    new_values['filter_model_ids'] = [(4, model_id) for model_id in model_ids]
    models_file.close()
execute('ir.module.module', 'write', module_ids, new_values)
data = execute('ir.module.module', 'get_relation_graph', module)
dot_file = open(module + '.dot', 'w')
dot_file.write(base64.decodestring(data['dot_graph']))
dot_file.close()
png_file = open(module + '.png', 'w')
png_file.write(base64.decodestring(data['file_graph']))
png_file.close()
execute('ir.module.module', 'write', module_ids, old_values)
