# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#    $Id: account.py 1005 2005-07-25 08:41:42Z nicoe $
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields

class hrself_department_wizard(osv.osv_memory):
    """Wizard to generate department based on levels."""

    _name = 'hrself.department.wizard'

    _columns = {
        'level_ids': fields.many2many('hrself.level', 'dept_level_rel', 'department_id', 'level_id', 'Levels'),
    }

    def _get_levels(self, cr, uid, context=None):
        """Levels used to generate departments."""
        result = []
        level_object = self.pool.get('hrself.level')
        level_ids = level_object.search(cr, uid, [], context=context)
        return level_ids

    _defaults = {
         'level_ids': _get_levels,
    }

    def create_departments(self, cr, uid, ids, context=None):
        """Create departments based on levels."""
        if context is None:
            context = {}

        wizard = self.browse(cr, uid, ids[0], context=context)
        level_object = self.pool.get('hrself.level')
        department_object = self.pool.get('hr.department')
        employee_object = self.pool.get('hr.employee')
        employee_ids = employee_object.search(cr, uid, [('user_id', '=', uid)])
        employee = employee_object.browse(cr, uid, employee_ids[0], context)
        universe_id = employee.universe_id.id
        user_object = self.pool.get('res.users')
        user = user_object.browse(cr, uid, uid, context)

        for level in wizard.level_ids:
            department_object.create(cr, uid, 
                { 'name': level_object.name_get(cr, uid, [level.id], context)[0][1],
                  'company_id': user.company_id.id,
                  'level_id': level.id,
                  'universe_id': employee.universe_id.id,
                }, context=context)

        return {
            'type': 'ir.actions.act_window_close',
        }

hrself_department_wizard()
