# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields

class res_company(osv.osv):
    """Company."""

    _inherit = "res.company"

    def synchro_get(self, cr, uid, ids, context=None):
        """Get the values useful for a synchronization.
        The interface of this method is similar to osv.orm.name_get."""
        return [(oid, False) for oid in ids]

    def synchro_search(self, cr, uid, values, context=None):
        """Search for records and their synchronization values according to values.
        The interface of this method is similar to osv.orm.name_search."""
        ids = self.search(cr, uid, [], context=context)
        return self.synchro_get(cr, uid, ids, context=context)

res_company()

class hrself_universe(osv.osv):
    """Universe."""

    _inherit = "hrself.universe"

    def synchro_get(self, cr, uid, ids, context=None):
        """Get the values useful for a synchronization.
        The interface of this method is similar to osv.orm.name_get."""
        return [(oid, False) for oid in ids]

    def synchro_search(self, cr, uid, values, context=None):
        """Search for records and their synchronization values according to values.
        The interface of this method is similar to osv.orm.name_search."""
        ids = self.search(cr, uid, [], context=context)
        return self.synchro_get(cr, uid, ids, context=context)

hrself_universe()

class hrself_employee_mapping(osv.osv):
    """Employee mapping."""

    _name = "hrself.employee.mapping"

    _columns = {
        'first_universe_id': fields.integer('First universe', required=True, help='First external universe identifier'),
        'first_employee_id': fields.integer('First employee', required=True, help='First external employee identifier'),
        'second_universe_id': fields.integer('Second universe', required=True, help='Second external universe identifier'),
        'second_employee_id': fields.integer('Second employee', required=True, help='Second external employee identifier'),
    }

hrself_employee_mapping()

class hr_employee(osv.osv):
    """Employee."""

    _inherit = "hr.employee"

    def synchro_get(self, cr, uid, ids, context=None):
        """Get the values useful for a synchronization.
        The interface of this method is similar to osv.orm.name_get."""
        return [(employee.id, (employee.universe_id.groupes_id, employee.groupes_id)) for employee in self.browse(cr, uid, ids, context=context)]

    def synchro_search(self, cr, uid, values, context=None):
        """Search for records and their synchronization values according to values.
        The interface of this method is similar to osv.orm.name_search."""
        ids = []
        mapping_object = self.pool.get('hrself.employee.mapping')
        mapping_ids = mapping_object.search(cr, uid, [
            ('first_universe_id', '=', values[0]),
            ('first_employee_id', '=', values[1]),
        ], context=context)
        if mapping_ids:
            mapping = mapping_object.browse(cr, uid, mapping_ids[0], context=context)
            universe_ids = self.pool.get('hrself.universe').search(cr, uid, [
                ('groupes_id', '=', mapping.second_universe_id),
            ], context=context)
            if universe_ids:
                ids = self.search(cr, uid, [
                    ('universe_id', '=', universe_ids[0]),
                    ('groupes_id', '=', mapping.second_employee_id),
                ], context=context)
        return self.synchro_get(cr, uid, ids, context=context)

hr_employee()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
